/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "Map.h"

using namespace std;
using namespace FD;

class CvfdhImageToMap;
   
DECLARE_NODE(CvfdhImageToMap)
/*Node
 *
 * @name CvfdhImageToMap
 * @category FDOPENCV_HARK:Misc
 * @description CvfdhImageToMap
 *
 * @input_name IMAGEIN
 * @input_description CvfdImage type input image
 * @input_type CvfdImage
 *
 * @output_name IMAGEOUT
 * @output_type Map<int,ObjectRef>
 * @output_description Map type output image
 *
 * @parameter_name NUM_COPIES
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description number of copies of input image
 *
 END*/
   
   
class CvfdhImageToMap : public BufferedNode {

  int m_imageInID;
  int m_imageOutID;
  int num_copies;
      
public:
  CvfdhImageToMap(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    m_imageOutID = addOutput("IMAGEOUT");
    num_copies = dereference_cast<int>(parameters.get("NUM_COPIES"));
    inOrder = true;
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   
    if(imagePtr->getImage() == NULL){
      (*(outputs[m_imageOutID].buffer))[count] = ObjectRef(imagePtr);
      return;
    }

    RCPtr<Map<int, ObjectRef> > outputMapPtr(new Map<int, ObjectRef>);
    for(int cnt = 0; cnt < num_copies; cnt++)    
      (*outputMapPtr)[cnt] = imagePtr;
    (*(outputs[m_imageOutID].buffer))[count] = outputMapPtr; 

  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhImageToMap)
};
