/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;

   
class CvfdLoadImage;
   
DECLARE_NODE(CvfdLoadImage)
/*Node
 *
 * @name CvfdLoadImage
 * @category FDOPENCV_ORG:Image
 * @description Load an image from a file
 *
 * @input_name FILENAME
 * @input_description The file name to load
 * @input_type string
 *
 * @parameter_name ISCOLOR
 * @parameter_type int
 * @parameter_value -1
 * @parameter_description Specifies colorness of the loaded image   
 *
 * @output_name IMAGE
 * @output_description The loaded image data structure
 * @output_type CvfdImage
 *
 END*/
   
   
class CvfdLoadImage : public BufferedNode {
  //Input ID
  int m_filenameID;
  //Output ID
  int m_imageID;
  //Parameters
  int m_iscolor;
      
public:
  CvfdLoadImage(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_filenameID = addInput("FILENAME");
    //add outputs
    m_imageID = addOutput("IMAGE");
    //Initialize parameters         
    m_iscolor = dereference_cast<int>(parameters.get("ISCOLOR"));
  }
      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    //Read the inputs
    RCPtr<String> filename = getInput(m_filenameID,count); 
         
    //Handle
    RCPtr<CvfdImage> imagePtr(new CvfdImage(*filename, m_iscolor));       
    out[count] = imagePtr;
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdLoadImage)
};
