/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdContours.h"
#include "CvfdImage.h"
#include "CvfdColor.h"

using namespace std;
using namespace FD;

class CvfdSelectContour;
   
DECLARE_NODE(CvfdSelectContour)
/*Node
 *
 * @name CvfdSelectContour
 * @category FDOPENCV_ORG:Contours
 * @description Point to the contour indicate by the index
 *
 * @input_name CONTOURSIN
 * @input_description The contour to handle
 * @input_type CvfdContours
 *
 * @input_name INDEX
 * @input_description index of the contour
 * @input_type int
 *
 * @output_name CONTOURSOUT
 * @output_description The result contour
 * @output_type CvfdContours
 *
 * @output_name NBELEMENTS
 * @output_description Number of elemtents in the selected contour
 * @output_type int
 *
 * @output_name BOOL
 * @output_description true if the next contour exist. If false, the first contour is return
 * @output_type bool
 *
 END*/
inline std::string CINT(int value)
{
  std::stringstream sstr;
  sstr << value;
  return(sstr.str());
}   
   
class CvfdSelectContour : public BufferedNode {
      
  //Input ID
  int m_contoursInID;
  int m_indexID;
  //Output ID
  int m_contoursOutID;
  int m_nbElementsID;
  int m_boolID;
      
public:
  CvfdSelectContour(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_contoursInID = addInput("CONTOURSIN");
    m_indexID = addInput("INDEX");
         
    //add outputs
    m_contoursOutID = addOutput("CONTOURSOUT");
    m_nbElementsID = addOutput("NBELEMENTS");
    m_boolID = addOutput("BOOL");
         
  }
      
  void calculate(int output_id, int count, Buffer &out)
  {           
    //Read the inputs
    RCPtr<CvfdContours> contoursPtr = getInput(m_contoursInID,count);
    RCPtr<Int> indexPtr = getInput(m_indexID,count);
    //Handle of the inputs
         
    int i=0;
    int total;
    bool Bool = true;
    CvfdContours* contours = new CvfdContours(&(*contoursPtr));
    if((contours->getContours()) != 0)
      {
	for(i=0; i<*(indexPtr); i++)
	  {            
	    contours->setContours(contours->getContours()->h_next);
	    if((contours->getContours()) == 0)
	      {
		Bool = false;
		total=0;
		contours->setContours(contours->getFirstContours());
		break;
	      } 
	    total = contours->getContours()->total;
	  }
      }
    else
      {
	Bool = false;
	total=0;
      } 
    (*(outputs[m_boolID].buffer))[count] = ObjectRef(Bool::alloc(Bool));
    (*(outputs[m_contoursOutID].buffer))[count] = ObjectRef(contours); 
    (*(outputs[m_nbElementsID].buffer))[count] = ObjectRef(Int::alloc(total));
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdSelectContour)
};
